# $OpenXM$
#############################################################################################
# 外部プログラム　hgm_ko_nc_fb と R コマンド optim による, FB 分布の最尤推定の計算
# hgm.nk.optim_astro_f(), hgm.nk.optim_mag_f() (2 次元), 
# hgm.nk.optim_sN_eM() より実行. (次元 N = 3,4,5,6,7, データ M = 1,2,3)
#############################################################################################

#############################################################################################
# R から Asir を呼び出して、ファイルを介してデータをやりとりするコマンド 
# example
# > hgm.nk.call_asir("fctr(x^5-1);");
# 1
# Calling the registered quit callbacks...done.
# Read 1 item
# [1] "[[1,1],[x-1,1],[x^4+x^3+x^2+x+1,1]]"
#############################################################################################
hgm.nk.call_asir <- function(command) {
	cat("if (access(\"tmp_asir_result\")) {\n   remove_file(\"tmp_asir_result\");\n} else {\n}\n", file="tmp_asir.rr");
	cat("output(\"tmp_asir_result\");", file="tmp_asir.rr", append=TRUE)
	cat(command, file="tmp_asir.rr", append=TRUE)
	cat("output();", file="tmp_asir.rr", append=TRUE)
	system("asir -f tmp_asir.rr")
	scan("tmp_asir_result", sep="\n", skip=1, what=character(0))	
}

hgm.nk.test_call_asir1 <- function() {
	hgm.nk.call_asir("load(\"kw.rr\");c1([-1,-2],[3,4]);")
}

hgm.nk.test_call_asir2 <- function() {
	hgm.nk.call_asir("fctr(x^100-1);")
}

##############################################################################################
# hgm_ko_nc_fb を用いた最適値の計算
##############################################################################################

# hgm_ko_nc_fb を R から呼び出して使う
# x : data (hgm_ko_nc_fb の入力データ
#     x11 x12 x13 ... x22 x23 ... x33 ... ... y1 y2 y3 ...に対応 
hgm.nk.call_tkin <- function(x) {
	arg <- " --1 "
	for (i in 1:length(x)) {
		num <- sprintf("%f ", x[i])
		arg <- paste(arg, num)	
	}
	com <- paste("hgm_ko_nc_fb ", arg)
	com <- paste(com, "> tmp_tkin")
	# print(com)
	system(com)
	result <- scan("tmp_tkin", sep="\n", what=character(0))	

	# v には積分の値とその微分の値たちが入る
	n <- grep("fbnd", result)
	v <- result[n+1]

	# "[\t 12266.6,\t 6149.72,\t 9938.98,\t 3859.76]" 
	# --> "c( 12266.6, 6149.72, 9938.98, 3859.76)" 
	v <- substr(v, 2, nchar(v)-1)
	v <- gsub("\t", "", v)
	v <- paste("c(", v)
	v <- paste(v, ")")

	# 文字列 v を R の命令として実行
	vect <- eval(parse(text=v))
	return(vect)
}

hgm.nk.test_call_tkin <- function() {
	return(hgm.nk.call_tkin(c(1,2,3,4,5)))
}

# hgm_ko_nc_fb を用いた計算
hgm.nk.astro_f <- function(x) {
	s11 <- 0.3119
	s12 <- 0.0292
	s13 <- 0.0707
	s22 <- 0.3605
	s23 <- 0.0462
	s33 <- 0.3276
	s1 <- -0.0063
	s2 <- -0.0054
	s3 <- -0.0762
	e <- exp(-s11*x[1]-s12*x[2]-s13*x[3]-s22*x[4]-s23*x[5]-s33*x[6]-s1*x[7]-s2*x[8]-s3*x[9])
	v <- hgm.nk.call_tkin(x)
	val <- e*v[1]
	print(val)
	print(x)
	return(val)
}

hgm.nk.test_astro_f <- function() {
	x <- c(-0.161, 0.3377/2.0, 0.2538, 1.1104/2.0, 0.6424/2.0, -0.0928, -0.019, -0.0162, -0.2286)
	return(hgm.nk.astro_f(x))
}

hgm.nk.bound_astro_f <- function(x) {
	big_val <- 10000
	lb <- c(-30, -30, -30, -30, -30, -30, -30, -30, -30)
	ub <- c(10, 10, 10, 10, 20, -0.01, -0.01, -0.001, 10)
	for (i in 1:length(x)) {
		if (lb[i] > x[i])
			return(big_val);
		if (ub[i] < x[i])	
			return(big_val);
	}		
	return(hgm.nk.astro_f(x))
}

hgm.nk.optim_astro_f <- function() {
	start_p <- c(0.1,0.1,1,1,1,-1,-1,-1,-1)
	return(optim(start_p, hgm.nk.bound_astro_f))
}

hgm.nk.mag_f <- function(x) {
	s11 <- 0.045
	s12 <- -0.075
	s13 <- 0.014
	s22 <- 0.921
	s23 <- -0.122
	s33 <- 0.034
	s1 <-  0.082
	s2 <- -0.959
	s3 <-  0.131
	e <- exp(-s11*x[1]-s12*x[2]-s13*x[3]-s22*x[4]-s23*x[5]-s33*x[6]-s1*x[7]-s2*x[8]-s3*x[9])
	v <- hgm.nk.call_tkin(x)
	val <- e*v[1]
	print(val)
	print(x)
	return(val)
}

hgm.nk.test_mag_f <- function() {
	x <- c(7.065,-0.032/2.0, 3.422/2.0, 5.339, 24.922/2.0, -13.693, 1.642, -31.99, 31.992)
	return(hgm.nk.mag_f(x))
}

hgm.nk.test_mag_f2 <- function() {
	x <- c(5.985, 8.478/2.0, 2.902/2.0, 6.869, 16.732/2.0, -12.853, 9.762, -28.77, 24.142)
	return(hgm.nk.mag_f(x))
}

hgm.nk.bound_mag_f <- function(x) {
	big_val <- 10000000000000000
	lb <- c(-30, -30, -30, -30, -30, -30, -30, -32, -30)
	ub <- c(30, 30, 30, 30, 30, -0.01, 30, -0.001, 32)
	for (i in 1:length(x)) {
		if (lb[i] > x[i])
			return(big_val);
		if (ub[i] < x[i])	
			return(big_val);
	}		
	return(hgm.nk.mag_f(x))
}

hgm.nk.optim_mag_f <- function() {
	start_p <- c(0.1,0.1,1,1,1,-1,-1,-1,-1)
	return(optim(start_p, hgm.nk.bound_mag_f))
}

##############################################################################################
# 3 次元 FB 積分 
##############################################################################################
hgm.nk.s3_e1 <- function(x) {
	s11 <- 0.155281954300000; 
	s12 <- -0.0135692760900000; 
	s13 <- -0.0694967727500000; 
	s14 <- -0.0189193035300000; 
	s22 <- 0.185126948900000; 
	s23 <- 0.00246151302000000; 
	s24 <- 0.127944099200000; 
	s33 <- 0.320347687400000; 
	s34 <- -0.0655441988700000; 
	s44 <- 0.339243422400000; 
	s1 <- 0.0642227387900000; 
	s2 <- 0.0806740628300000; 
	s3 <- 0.152724970200000; 
	s4 <- -0.199412384400000;
	e <- exp(-s11*x[1]-s12*x[2]-s13*x[3]-s14*x[4]-s22*x[5]-s23*x[6]-s24*x[7]-s33*x[8]-s34*x[9]-s44*x[10]-s1*x[11]-s2*x[12]-s3*x[13]-s4*x[14]);
	v <- hgm.nk.call_tkin(x)
	print(v)
	val <- e*v[1]
	print(val)
	return(val)
}

hgm.nk.test_s3_e1 <- function() {
	x <- c(-1.56641, -0.660658, -1.31266, 0.0264586, -2.18922, 0.00632535, 2.64592, 0.967726, -0.474077, 0.276654, 0.906395, 1.63355, 0.571844, -1.26449)
	return(hgm.nk.s3_e1(x))
}

hgm.nk.optim_s3_e1 <- function() {
	start_p <- seq(0.1, 1.4, 0.1)
	return(optim(start_p, hgm.nk.s3_e1))
}

hgm.nk.s3_e2 <- function(x) {
	s11 <- 0.218092016000000; 
	s12 <- 0.0588335768900000; 
	s13 <- 0.0771857690000000; 
	s14 <- 0.169482099700000; 
	s22 <- 0.290852319600000; 
	s23 <- -0.114649936000000; 
	s24 <- 0.0384662736300000; 
	s33 <- 0.225877156900000; 
	s34 <- 0.115050164700000; 
	s44 <- 0.265178513600000; 
	s1  <- -0.134022828300000; 
	s2  <- 0.303831075800000; 
	s3  <- -0.258739873300000; 
	s4  <- -0.165112242500000;
	e <- exp(-s11*x[1]-s12*x[2]-s13*x[3]-s14*x[4]-s22*x[5]-s23*x[6]-s24*x[7]-s33*x[8]-s34*x[9]-s44*x[10]-s1*x[11]-s2*x[12]-s3*x[13]-s4*x[14]);
	v <- hgm.nk.call_tkin(x)
	print(v)
	val <- e*v[1]
	print(val)
	return(val)
}

hgm.nk.optim_s3_e2 <- function() {
	start_p <- seq(0.1, 1.4, 0.1)
	return(optim(start_p, hgm.nk.s3_e2))
}

hgm.nk.s3_e3 <- function(x) {
	s11 <- 0.151575175700000; 
	s12 <- 0.0427121331300000; 
	s13 <- 0.0314851677100000; 
	s14 <- -0.00420459559500000; 
	s22 <- 0.356894309000000; 
	s23 <- -0.00414464625600000; 
	s24 <- -0.0192891821000000; 
	s33 <- 0.316517313200000; 
	s34 <- -0.0355512305700000; 
	s44 <- 0.175013205600000; 
	s1 <- 0.151976432700000; 
	s2 <- 0.120828095300000; 
	s3 <- -0.0799170663700000; 
	s4 <- 0.199418003900000; 
	e <- exp(-s11*x[1]-s12*x[2]-s13*x[3]-s14*x[4]-s22*x[5]-s23*x[6]-s24*x[7]-s33*x[8]-s34*x[9]-s44*x[10]-s1*x[11]-s2*x[12]-s3*x[13]-s4*x[14]);
	v <- hgm.nk.call_tkin(x)
	print(v)
	val <- e*v[1]
	print(val)
	return(val)
}

hgm.nk.optim_s3_e3 <- function() {
	start_p <- seq(0.1, 1.4, 0.1)
	return(optim(start_p, hgm.nk.s3_e3))
}

##############################################################################################
# 4 次元 FB 積分 
##############################################################################################
hgm.nk.s4_e1 <- function(x) {
	s <- c(0.172459497100000 , 0.0576460972800000 , 0.0307086298200000 , 0.00893655529400000 , -0.0557141367600000, 0.229190050100000 , -0.00263331345500000 , 0.0000624262210700000 , 0.0319320576500000, 0.188279064400000 , -0.0437653004700000 , 0.0634128653100000, 0.177884123000000 , -0.0533448297200000, 0.232187270900000, -0.0326839313800000, -0.136571153500000, -0.0248075723600000, -0.170649362200000, 0.0545479796500000) 
	e <- 1
	for (i in 1:length(s)) {
		e <- e * exp(-s[i] * x[i])
	}
	v <- hgm.nk.call_tkin(x)
	print(v)
	val <- e*v[1]
	print(val)
	return(val)
}

hgm.nk.test_s4_e1 <- function() {
	p <- c(-0.931429, 1.64605, 1.43581, 0.348762, -1.72137, 0.501904, -0.712407, -0.471698, 1.0895, -0.273174, -1.15294, 1.44572, -0.711815, -0.58341, 0.414167, 0.418325, -0.931711, -0.722413, -1.38142, 0.380943)
	return(hgm.nk.s4_e1(p))	
}

hgm.nk.optim_s4_e1 <- function() {
	start_p <- seq(0.1, 2.0, 0.1)
	return(optim(start_p, hgm.nk.s4_e1))
}

hgm.nk.s4_e2 <- function(x) {
	s <- c(0.159297942900000 , -0.119311098400000 , -0.0649843366700000 , -0.0881004685200000 , 0.0336464337500000, 0.232132493800000 , 0.101052411800000 , 0.102283772200000 , -0.0144719066200000, 0.139118584100000 , -0.00150942001700000 , -0.0554461831200000, 0.284087860200000 , 0.0599028828700000, 0.185363112700000, 0.0347357177100000, 0.0336302980200000, 0.142509686000000, -0.285357104900000, -0.231432532400000)
	e <- 1
	for (i in 1:length(s)) {
		e <- e * exp(-s[i] * x[i])
	}
	v <- hgm.nk.call_tkin(x)
	print(v)
	val <- e*v[1]
	print(val)
	return(val)
}

hgm.nk.optim_s4_e2 <- function() {
	start_p <- seq(0.1, 2.0, 0.1)
	return(optim(start_p, hgm.nk.s4_e2))
}

hgm.nk.s4_e3 <- function(x) {
	s <- c(0.149502249500000 , 0.0941522574400000 , -0.00596860990500000 , -0.0600232905600000 , 0.0552994152000000, 0.284592950100000 , -0.0170736801200000 , -0.0760050403200000 , 0.0120812241100000, 0.162358296100000 , 0.0438103943600000 , -0.0339864493800000, 0.223043344300000 , -0.0380870802100000, 0.180503141900000, -0.204925954800000, -0.314438322100000, 0.122148857300000, 0.235622148000000, -0.179881077400000)
	e <- 1
	for (i in 1:length(s)) {
		e <- e * exp(-s[i] * x[i])
	}
	v <- hgm.nk.call_tkin(x)
	print(v)
	val <- e*v[1]
	print(val)
	return(val)
}

hgm.nk.optim_s4_e3 <- function() {
	start_p <- seq(0.1, 2.0, 0.1)
	return(optim(start_p, hgm.nk.s4_e3))
}

##############################################################################################
# 5 次元 FB 積分 
##############################################################################################
hgm.nk.s5_e1 <- function(x) {
	s <- c(0.128413963400000 , 0.0280009922600000 , 0.00441926145400000 , -0.0638585037000000 , 0.0267388406700000 , 0.0506288583000000, 0.158654840900000 , -0.00486396171100000, 0.0401815975900000 , -0.0204032645900000 , -0.0976875966500000, 0.127047053400000, 0.0178488378500000 , -0.0511339108400000 , 0.0459847338100000, 0.177713273500000 , -0.0410542115500000 , -0.0789339750000000, 0.175852060000000 , 0.0494083174000000, 0.232318818300000, -0.0107115657300000, -0.179419239100000, -0.0500537639500000, -0.143512281900000, -0.0850789167200000, 0.108370196100000)
	e <- 1
	for (i in 1:length(s)) {
		e <- e * exp(-s[i] * x[i])
	}
	v <- hgm.nk.call_tkin(x)
	print(v)
	val <- e*v[1]
	print(val)
	return(val)
}

hgm.nk.test_s5_e1 <- function() {
	p <- c(-2.16521, 2.79454, 0.042826, -2.4867, 0.407213, 1.95608, -1.80309, -0.135131, 0.673371, -0.852351, -2.87539, -1.85027, 0.571216, -2.96803, 2.38851, -0.249028, -0.849152, -0.943944, -0.336686, 1.51726, 0.162898, -0.133927, -1.54449, -1.36794, -1.08446, -1.75098, 0.34225)
	return(hgm.nk.s5_e1(p))	
}

hgm.nk.optim_s5_e1 <- function() {
	start_p <- seq(0.01, 0.27, 0.01)
	return(optim(start_p, hgm.nk.s5_e1))
}

hgm.nk.s5_e2 <- function(x) {
	s <- c(0.193814929700000 , 0.0377082399200000 , -0.107959875300000 , 0.0226327651700000 , -0.0627727558000000 , 0.111970418200000, 0.146891460700000 , -0.0422004872900000 , 0.0411029074100000 , 0.0495995367600000 , 0.00944967086900000, 0.207225289900000 , 0.0175019685900000 , 0.0521413642400000 , -0.0777256242200000, 0.114288796400000 , 0.0347437943200000 , 0.0329221144000000, 0.141939392300000 , -0.0566384434200000, 0.195840143100000, -0.251019266800000, -0.216706617600000, 0.182651671400000, -0.100455261200000, 0.00290129459800000, -0.154256838700000)
	e <- 1
	for (i in 1:length(s)) {
		e <- e * exp(-s[i] * x[i])
	}
	v <- hgm.nk.call_tkin(x)
	print(v)
	val <- e*v[1]
	print(val)
	return(val)
}

hgm.nk.optim_s5_e2 <- function() {
	start_p <- seq(0.01, 0.27, 0.01)
	return(optim(start_p, hgm.nk.s5_e2))
}

hgm.nk.s5_e3 <- function(x) {
	s <- c(0.165345468100000 , -0.0107934200200000 , 0.112178178800000 , 0.00921465661600000 , -0.0359058550600000 , -0.00222486010300000, 0.194117008600000 , 0.0285278175000000 , 0.0407127505700000 , -0.00177417332800000 , -0.0619162581800000, 0.261324357200000 , 0.0242544666700000 , -0.0278821166800000 , -0.0328207597500000, 0.160570991300000 , -0.0529857833100000 , -0.0160180022200000, 0.114683429100000 , 0.0140282836300000, 0.103958752500000, -0.176212529000000, -0.203568916600000, -0.263959979200000, 0.0104231691200000, 0.0718219008100000, 0.138741541600000)
	e <- 1
	for (i in 1:length(s)) {
		e <- e * exp(-s[i] * x[i])
	}
	v <- hgm.nk.call_tkin(x)
	print(v)
	val <- e*v[1]
	print(val)
	return(val)
}

hgm.nk.optim_s5_e3 <- function() {
	start_p <- seq(0.01, 0.27, 0.01)
	return(optim(start_p, hgm.nk.s5_e3))
}

##############################################################################################
# 6 次元 FB 積分 
##############################################################################################
hgm.nk.s6_e1 <- function(x) {
	s <- c(0.281082925000000 , -0.124299957700000 , 0.00897572574600000 , -0.0743553338900000 , -0.123589530000000 , 0.0168754789500000 , -0.0678267898900000, 0.152573805900000 , 0.00845490929800000 , 0.0476652208900000 , 0.0976166335100000 , 0.0220196995000000 , 0.0566712103600000, 0.112705126400000 , 0.00694563347000000 , 0.0378003405800000 , 0.0252070968900000 , 0.00633786782900000, 0.0818649759000000 , 0.0444584168700000 , 0.0159262131300000 , 0.000932959264500000, 0.156627834900000 , 0.0280033918300000 , 0.0584549596200000, 0.0963216608100000 , -0.0220847523400000, 0.118823669800000, -0.0827478176000000, 0.0212003013000000, 0.0988723883900000, 0.0641611119400000, 0.115150890800000, 0.103436330000000, -0.0755932901200000)
	e <- 1
	for (i in 1:length(s)) {
		e <- e * exp(-s[i] * x[i])
	}
	v <- hgm.nk.call_tkin(x)
	print(v)
	val <- e*v[1]
	print(val)
	return(val)
}

hgm.nk.test_s6_e1 <- function() {
	p <- c(2.95733, -2.17945, 1.14686, -2.34062, -2.47342, 2.18745, -0.750356, -0.499759, -0.254907, 1.62409, 2.59485, 1.95868, 1.70849, 1.055, 0.330472, 1.83485, 0.509892, 0.599487, -2.51101, 0.483319, 1.04527, -1.66155, -1.44693, 1.87403, 2.78653, -1.26776, -1.85351, -0.594946, -0.611049, -0.844609, 0.846312, 0.23329, 1.13458, 1.01146, -1.26431) 
	return(hgm.nk.s6_e1(p))	
}

hgm.nk.optim_s6_e1 <- function() {
	start_p <- seq(0.01, 0.35, 0.01)
	return(optim(start_p, hgm.nk.s6_e1))
}

##############################################################################################
# 7 次元 FB 積分 
##############################################################################################
hgm.nk.s7_e1 <- function(x) {
	s <- c(0.144234708800000 , -0.0307354144200000 , 0.0475007164100000 , 0.0375383538000000 , 0.0204359762500000 , 0.0132516125900000 , -0.0208749802900000 , 0.0316842915900000, 0.126718381900000 , 0.0438997010600000 , 0.0317871988000000 , -0.0251460250300000 , 0.00836762032000000 , 0.00720307180500000 , -0.0295623926300000, 0.177136351700000 , 0.0997703831100000 , 0.0000205554400000000 , -0.0167840384100000 , -0.0645375603700000 , 0.00418221698900000, 0.155543232900000 , -0.0142516803300000 , -0.0510239993500000 , -0.0593951823200000 , -0.00300960325700000, 0.0929358430000000 , 0.0401190739700000 , -0.0123088957100000 , 0.0165724158900000, 0.126210284600000 , 0.00971771855200000 , 0.00152110498400000, 0.0966718168200000 , -0.00267139564900000, 0.0805493756300000, 0.0653533910600000, -0.00299983584800000, -0.0397389635500000, -0.0975975501600000, -0.0203241640900000, 0.0114251527700000, 0.103787469100000, 0.00749989928200000)
	e <- 1
	for (i in 1:length(s)) {
		e <- e * exp(-s[i] * x[i])
	}
	v <- hgm.nk.call_tkin(x)
	print(v)
	val <- e*v[1]
	print(val)
	return(val)
}

hgm.nk.test_s7_e1 <- function() {
	p <- c(1.06063, -2.01636, 1.44765, 2.07161, 0.36159, 1.59727, 0.126302, 1.45987, -0.0793436, 2.36966, 1.77493, -1.49226, 1.8733, 1.89958, -1.5444, 0.55414, 2.71741, 0.343879, -0.259269, -2.8271, 0.624258, -0.997604, -0.349956, -2.87397, -2.34222, -0.544477, -1.05343, 2.57101, -0.909933, 0.912665, 0.18692, -0.288584, -0.416114, -2.8334, 0.427543, -1.30137, 1.00287, 0.155828, 0.342829, -1.02349, -0.199391, -0.432356, 1.10098, -0.199029);
	return(hgm.nk.s7_e1(p))	
}

hgm.nk.optim_s7_e1 <- function() {
	start_p <- seq(0.01, 0.44, 0.01)
	return(optim(start_p, hgm.nk.s7_e1))
}
